#!/usr/bin/env python

"""
XSL output classes and functions.
"""

__version__ = "0.1"

# NOTE: Make this use other implementations, too.

import libxsltmod
import libxml2dom

class OutputError(Exception):
    pass

class Processor:

    """
    A handler which can prepare output for an XMLTools2 template.
    """

    def __init__(self, filenames, references=None, parameters=None):

        """
        Initialise the handler with the 'filenames' of stylesheets producing the
        final output, a 'references' dictionary indicating related stylesheets.
        Additional 'parameters' may also be specified as a dictionary.
        """

        self.references = references or {}
        self.parameters = parameters or {}

        # Remember the stylesheet documents.

        self.stylesheets = []
        for filename in filenames:
            self.stylesheets.append(libxsltmod.xsltParseStylesheetFile(filename))

    def __del__(self):

        """
        Tidy up the stylesheet documents.
        """

        for stylesheet in self.stylesheets:
            libxsltmod.xsltFreeStylesheet(stylesheet)

    def send_output(self, stream, encoding, document):

        """
        Send output to the given 'stream' using the given output encoding for
        the given 'document'.
        """

        result = self.get_result(document)

        if result is not None:
            stream.write(result.toString(encoding))
        else:
            raise OutputError, "Transformation failed."

    def get_result(self, document):

        """
        Return a transformed document produced from the object's stylesheets and
        the given 'document'.
        """

        result = self._get_result(document)

        if result is not None:
            return libxml2dom.adoptNodes([result])[0]
        else:
            raise OutputError, "Transformation failed."

    def _get_result(self, document):

        """
        Return a transformation of the given 'document'.
        """

        if hasattr(document, "as_native_node"):
            document = document.as_native_node()

        # Transform the localised instance into the final output.

        parameters = {}
        for name, reference in self.references.items():
            parameters[name.encode("utf-8")] = ("document('%s')" % self._quote(reference)).encode("utf-8")
        for name, parameter in self.parameters.items():
            parameters[name.encode("utf-8")] = ("'%s'" % self._quote(parameter)).encode("utf-8")
        #print "**", repr(parameters)

        last_result = document
        for stylesheet in self.stylesheets:
            result = libxsltmod.xsltApplyStylesheet(stylesheet, last_result, parameters)
            if last_result is not None:
                last_result = result
            else:
                raise OutputError, "Transformation failed."

        return result

    def _quote(self, s):

        "Make the given parameter string 's' palatable for libxslt."

        return s.replace("'", "%27")

# vim: tabstop=4 expandtab shiftwidth=4
